@@ -35,10 +35,10 @@ class LoggerStatus(AutoStrEnum):
 
 
 default_key_word_success = ['success', 'ok', 'done', 'yes', 'okay', 'true', 'complete', 'pass']
-default_key_word_fails = ['fail', 'false', 'error', 'bug']
-default_color_list = [Fore.GREEN, Fore.YELLOW, Fore.BLUE, Fore.MAGENTA, Fore.CYAN]
+default_key_word_fails = ['fail', 'false', 'error', 'bug', 'fire']
+default_color_list = [Fore.YELLOW, Fore.BLUE, Fore.MAGENTA, Fore.CYAN, Fore.LIGHTYELLOW_EX, Fore.LIGHTBLUE_EX,
+                      Fore.LIGHTMAGENTA_EX, Fore.LIGHTCYAN_EX]
 last_logger: [Logger | None] = None
-is_first_print = True
 
 
 def new_print(*args, **kwargs):
@@ -53,7 +53,7 @@ BOLD = '\033[1m'
 class Logger:
 
     def __init__(self, log_name: [str | None] = 'logger', log_level: LogLevel = LogLevel.INFO,
-                 skip_repeat: bool = False, handler: [Callable | List[Callable]] = None, stage_sep: str = '...',
+                 skip_repeat: bool = False, handler: [Callable | List[Callable]] = None, stage_sep: str = '..',
                  timestamp: [str | None] = "%m.%d %H:%M:%S", key_word_success: [list | None] = None,
                  key_word_fails: [list | None] = None, stage_color_list: [List[Fore] | None] = None):
         """
@@ -110,8 +110,7 @@ class Logger:
 
         self.status = LoggerStatus.FINISH
         self._last_msg = None
-        self._do_level = None
-        self.check_add_new_line = False
+        self._stage_level = None
 
     def info(self, *msg):
         self._start_check(LogLevel.INFO, *msg)
@@ -124,25 +123,30 @@ class Logger:
 
     def _start_check(self, log_level: LogLevel, *msg):
 
-        # old_print(f'start {self.status}', end='')
-        if self.status != LoggerStatus.FINISH:
-            self.check_add_new_line = True
-        self.status = LoggerStatus.START
-        if self._start(log_level, *msg):
-            self.status = LoggerStatus.STAGE
-            self._do_level = log_level
-        elif not self.check_add_new_line:
-            self.status = LoggerStatus.FINISH
+        with global_lock:
+            if self._start(log_level, *msg):
+                self.status = LoggerStatus.STAGE
+                self._stage_level = log_level
 
     def stage(self, *msg):
         # its log level is the same as the last do_level
         if self.status == LoggerStatus.FINISH:
             # works like normal logger
             self._start_check(LogLevel.INFO, *msg)
-        elif self.status in [LoggerStatus.STAGE, LoggerStatus.START]:
-            self._stage(self._do_level, *msg)
-        else:
-            raise Exception(f'Unknown log status {self.status}')
+            return
+
+        self._stage(self._stage_level, *msg)
+
+    def _print(self, *args, **kwargs):
+        with global_lock:
+            self.status = LoggerStatus.PRINT
+            self._add_newline()
+            self._output(None, *args, **kwargs)
+            self.status = LoggerStatus.FINISH
+
+            global last_logger
+            last_logger = self
+
 
     def _check_log_level(self, log_level: LogLevel) -> bool:
         # check the msg will be output or not
@@ -157,12 +161,14 @@ class Logger:
 
     def _stage(self, log_level: LogLevel, msg):
         if not self._check_log_level(log_level):
-            return False
+            return
 
         if self.status not in [LoggerStatus.STAGE, LoggerStatus.START]:
             raise Exception(f'Unknown log status {self.status}')
 
         message = str(msg)
+        if self.skip_repeat:
+            self._last_msg = message
 
         color = ''
         for s in self.key_word_success:
@@ -188,15 +194,13 @@ class Logger:
         global last_logger
         last_logger = self
 
-    def _print(self, *args, **kwargs):
-
-        self._lock_area(None, *args, **kwargs)
-
     def _start(self, log_level: LogLevel, *msg) -> bool:
 
         if not self._check_log_level(log_level):
             return False
 
+        self.status = LoggerStatus.START
+
         if not msg:
             msg = ' '
 
@@ -220,44 +224,53 @@ class Logger:
         timestamp = f'[{strftime(self.timestamp)}]' if self.timestamp else ''
         total_message = f'{timestamp}{self.log_name}{location} {message}'.strip()
 
-        self._lock_area(total_message)
+        self._add_newline()
+        self._output(total_message)
+
+        global last_logger
+        last_logger = self
 
         return True
 
-    def _lock_area(self, total_message: [str | None], *args, **kwargs):
-        with global_lock:
-            global is_first_print
-            global last_logger
+    def __add_newline(self) -> None:
+        old_print()
+        if last_logger is not self:
+            last_logger.status = LoggerStatus.FINISH
+        self._stage_count = 0
 
-            if not is_first_print:
-                if self.check_add_new_line:
-                    self.check_add_new_line = False
-                    old_print()
-                    last_logger.status = LoggerStatus.FINISH
-                elif self.status == LoggerStatus.START:
-                    if last_logger.status != LoggerStatus.FINISH:
-                        old_print()
-                else:
-                    old_print()
-            else:
-                is_first_print = False
+    def _add_newline(self):
 
-            last_logger = self
-            if self.status == LoggerStatus.PRINT:
-                kwargs['end'] = ''
-                old_print(*args, **kwargs)
+        if self.status == LoggerStatus.STAGE:
+            # if the last stage is stage, don't need to lock
+            raise Exception('Cannot print in stage status')
 
-                return True
+        global last_logger
+        if last_logger:
+            if self.status == LoggerStatus.START:
+                # if the status is start, it means we need to check the status of last logger
+                # note: last logger could be self
+                if last_logger.status != LoggerStatus.FINISH:
+                    # if self or last logger is not finish, add newline
+                    self.__add_newline()
+            elif self.status == LoggerStatus.PRINT:
+                # is the last print is not print, we need to add newline
+                # adjust this to avoid the situation that the last print is print
+                if last_logger.__class__.__name__ != 'PrintLogger':
+                    # if self is not last logger, add newline
+                    self.__add_newline()
 
-            utils.output_screen(total_message)
-            utils.output_file(self.handlers, total_message)
+    def _output(self, total_message: [str | None], *args, **kwargs) -> None:
+        if self.status == LoggerStatus.PRINT:
+            old_print(*args, **kwargs)
+            return
+
+        utils.output_screen(total_message)
+        utils.output_file(self.handlers, total_message)
 
-            return True
 
 
 class PrintLogger(Logger):
     def print(self, *args, **kwargs):
-        self.status = LoggerStatus.PRINT
         self._print(*args, **kwargs)
 
 
